@@ -2,21 +2,23 @@
 *									       *
 * textDrag.c - Text Dragging routines for NEdit text widget		       *
 *									       *
-* Copyright (c) 1995 Universities Research Association, Inc.		       *
-* All rights reserved.							       *
+* Copyright (C) 1999 Mark Edel						       *
+*									       *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version.							               *
 * 									       *
-* This material resulted from work developed under a Government Contract and   *
-* is subject to the following license:  The Government retains a paid-up,      *
-* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
-* works, perform publicly and display publicly by or for the Government,       *
-* including the right to distribute to other Government contractors.  Neither  *
-* the United States nor the United States Department of Energy, nor any of     *
-* their employees, makes any warranty, express or implied, or assumes any      *
-* legal liability or responsibility for the accuracy, completeness, or         *
-* usefulness of any information, apparatus, product, or process disclosed, or  *
-* represents that its use would not infringe privately owned rights.           *
-*                                        				       *
-* Fermilab Nirvana GUI Library						       *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* for more details.							       *
+* 									       *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA		                       *
+*									       *
+* Nirvana Text Editor	    						       *
 * Dec. 15, 1995								       *
 *									       *
 * Written by Mark Edel							       *
@@ -58,7 +60,7 @@ void BeginBlockDrag(TextWidget tw)
     int fontHeight = textD->fontStruct->ascent + textD->fontStruct->descent;
     int fontWidth = textD->fontStruct->max_bounds.width;
     selection *sel = &buf->primary;
-    int nLines, mousePos, mouseLineNum, lineStart;
+    int nLines, mousePos, lineStart;
     int x, y, lineEnd;
     
     char *text;
@@ -82,18 +84,16 @@ void BeginBlockDrag(TextWidget tw)
        non-rectangular selections)  */
     if (sel->rectangular) {
     	tw->text.dragXOffset = tw->text.btnDownX + textD->horizOffset -
-    	    	tw->text.marginWidth - sel->rectStart * fontWidth;
+    	    	textD->left - sel->rectStart * fontWidth;
     } else {
         if (!TextDPositionToXY(textD, sel->start, &x, &y))
-            x = BufCountDispChars(buf, BufStartOfLine(buf, sel->start),
-            	    sel->start) * fontWidth + tw->text.marginWidth -
+            x = BufCountDispChars(buf, TextDStartOfLine(textD, sel->start),
+            	    sel->start) * fontWidth + textD->left -
             	    textD->horizOffset;
         tw->text.dragXOffset = tw->text.btnDownX - x;
     }
     mousePos = TextDXYToPosition(textD, tw->text.btnDownX, tw->text.btnDownY);
     nLines = BufCountLines(buf, sel->start, mousePos);
-    if (!TextDPosToLineNum(textD, mousePos, &mouseLineNum))
-    	fprintf(stderr, "NEdit, internal consistency check bbd failed\n");
     tw->text.dragYOffset = nLines * fontHeight + (((tw->text.btnDownY -
     	    tw->text.marginHeight) % fontHeight) - fontHeight/2);
     tw->text.dragNLines = BufCountLines(buf, sel->start, sel->end);
@@ -277,9 +277,13 @@ void BlockDragSelection(TextWidget tw, int x, int y, int dragType)
     trackModifyRange(&modRangeStart, &bufModRangeEnd, &tempModRangeEnd,
     	   tw->text.dragInsertPos, tw->text.dragInserted, tw->text.dragDeleted);
     
-    /* Find the line number of the insert position */
+    /* Find the line number and column of the insert position.  Note that in
+       continuous wrap mode, these must be calculated as if the text were
+       not wrapped */
     TextDXYToUnconstrainedPosition(textD, max(0, x - dragXOffset),
     	    max(0, y - (tw->text.dragYOffset % fontHeight)), &row, &column);
+    column = TextDOffsetWrappedColumn(textD, row, column);
+    row = TextDOffsetWrappedRow(textD, row);
     insLineNum = row + textD->topLineNum - tw->text.dragYOffset / fontHeight;
     
     /* find a common point of reference between the two buffers, from which
@@ -360,7 +364,7 @@ void BlockDragSelection(TextWidget tw, int x, int y, int dragType)
     tw->text.dragSourceInserted = sourceInserted;
     tw->text.dragSourceDeleted = sourceDeleted;
     tw->text.dragType = dragType;
-    
+ 
     /* Reset the selection and cursor position */
     if (rectangular || overlay) {
     	insRectEnd = insRectStart + origSel->rectEnd - origSel->rectStart;
@@ -524,9 +528,11 @@ static void findTextMargins(textBuffer *buf, int start, int end, int *leftMargin
     	    width = 0;
     	    inWhite = True;
     	} else
-    	    width += BufCharWidth(c, width, buf->tabDist);
+    	    width += BufCharWidth(c, width, buf->tabDist, buf->nullSubsChar);
     }
-    *leftMargin = minWhite;
+    if (width > maxWidth)
+    	maxWidth = width;
+    *leftMargin = minWhite == INT_MAX ? 0 : minWhite;
     *rightMargin = maxWidth;
 }
 
