@@ -1,29 +1,29 @@
 /*******************************************************************************
 *									       *
-* fontsel.c -- Nirvana Font Selector main program			       *
+* fontsel.c -- Nirvana Font Selector			       *
 *									       *
-* Copyright (c) 1993 Universities Research Association, Inc.		       *
-* All rights reserved.							       *
+* Copyright (C) 1999 Mark Edel						       *
+*									       *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version.							               *
 * 									       *
-* This material resulted from work developed under a Government Contract and   *
-* is subject to the following license:  The Government retains a paid-up,      *
-* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
-* works, perform publicly and display publicly by or for the Government,       *
-* including the right to distribute to other Government contractors.  Neither  *
-* the United States nor the United States Department of Energy, nor any of     *
-* their employees, makes any warrenty, express or implied, or assumes any      *
-* legal liability or responsibility for the accuracy, completeness, or         *
-* usefulness of any information, apparatus, product, or process disclosed, or  *
-* represents that its use would not infringe privately owned rights.           *
-*                                        				       *
-* Fermilab Nirvana GUI Library						       *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* for more details.							       *
+* 									       *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA		                       *
+*									       *
+* Nirvana Text Editor	    						       *
 * June 2, 1993								       *
 *									       *
 * Written by Suresh Ravoor (assisted by Mark Edel)			       *
 *									       *
 *******************************************************************************/
-static char SCCSID[] = "@(#)fontsel.c	1.12	10/3/96";
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <X11/Intrinsic.h>
@@ -86,9 +86,9 @@ typedef struct
 static void	getStringComponent(char *inStr, int pos, char *outStr);
 static void	setupScrollLists(int dontChange, xfselControlBlkType ctrlBlk);
 static int	notPropFont(char *font);
-static int	styleMatch(xfselControlBlkType ctrlBlk, char *font);
-static int	sizeMatch(xfselControlBlkType ctrlBlk, char *font);
-static int	fontMatch(xfselControlBlkType ctrlBlk, char *font);
+static int	styleMatch(xfselControlBlkType *ctrlBlk, char *font);
+static int	sizeMatch(xfselControlBlkType *ctrlBlk, char *font);
+static int	fontMatch(xfselControlBlkType *ctrlBlk, char *font);
 static void	addItemToList(char **buf, char *item, int *count);
 static void	getFontPart(char *font, char *buff1);
 static void	getStylePart(char *font, char *buff1);
@@ -336,8 +336,8 @@ char 	*FontSel(Widget parent, int showPropFonts, char *currFont)
 	XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg(args[n], XmNtopWidget, nameLabel); n++;
 	XtSetArg(args[n], XmNleftOffset, 5); n++;
-	XtSetArg(args[n], XmNleftWidget, fontList); n++;
-	XtSetArg(args[n], XmNbottomWidget, fontList); n++;
+	XtSetArg(args[n], XmNleftWidget, XtParent(fontList)); n++;
+	XtSetArg(args[n], XmNbottomWidget, XtParent(fontList)); n++;
 	XtSetArg(args[n], XmNrightPosition, 85); n++;
 	styleList = XmCreateScrolledList(form, "stylelist", args, n);
 	XtManageChild(styleList);
@@ -349,8 +349,8 @@ char 	*FontSel(Widget parent, int showPropFonts, char *currFont)
 	XtSetArg(args[n], XmNlabelString, tempStr); n++;
 	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
-	XtSetArg(args[n], XmNbottomWidget, styleList); n++;
-	XtSetArg(args[n], XmNleftWidget, styleList); n++;
+	XtSetArg(args[n], XmNbottomWidget, XtParent(styleList)); n++;
+	XtSetArg(args[n], XmNleftWidget, XtParent(styleList)); n++;
 	XtCreateManagedWidget("Style:", xmLabelWidgetClass, form, args, n);
 	XmStringFree(tempStr);
 
@@ -362,8 +362,8 @@ char 	*FontSel(Widget parent, int showPropFonts, char *currFont)
 	XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
 	XtSetArg(args[n], XmNtopWidget, nameLabel); n++;
-	XtSetArg(args[n], XmNleftWidget, styleList); n++;
-	XtSetArg(args[n], XmNbottomWidget, fontList); n++;
+	XtSetArg(args[n], XmNleftWidget, XtParent(styleList)); n++;
+	XtSetArg(args[n], XmNbottomWidget, XtParent(fontList)); n++;
 	XtSetArg(args[n], XmNleftOffset, 5); n++;
 	XtSetArg(args[n], XmNrightPosition, 99); n++;
 	sizeList = XmCreateScrolledList(form, "sizelist", args, n);
@@ -376,8 +376,8 @@ char 	*FontSel(Widget parent, int showPropFonts, char *currFont)
 	XtSetArg(args[n], XmNuserData, sizeList); n++;
 	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
-	XtSetArg(args[n], XmNbottomWidget, sizeList); n++;
-	XtSetArg(args[n], XmNleftWidget, sizeList); n++;
+	XtSetArg(args[n], XmNbottomWidget, XtParent(sizeList)); n++;
+	XtSetArg(args[n], XmNleftWidget, XtParent(sizeList)); n++;
 	XtCreateManagedWidget("Size:", xmLabelWidgetClass, form, args, n);
 	XmStringFree(tempStr);
 
@@ -409,7 +409,7 @@ char 	*FontSel(Widget parent, int showPropFonts, char *currFont)
 	ctrlBlk.sel3		= NULL;
 	ctrlBlk.fontName	= NULL;
 
- 	setupScrollLists(NONE, ctrlBlk);	/* update scroll lists */ 
+	setupScrollLists(NONE, ctrlBlk);	/* update scroll lists */ 
 
 	if (showPropFonts == PREF_PROP)
 		XmToggleButtonSetState(propFontToggle, TRUE, FALSE); 
@@ -531,8 +531,8 @@ static void	setupScrollLists(int dontChange, xfselControlBlkType ctrlBlk)
 	for (i = 0; i < ctrlBlk.numFonts && i < MAX_ENTRIES_IN_LIST; i++)
 	{
 		if ((dontChange != FONT) &&
-		    (styleMatch(ctrlBlk, ctrlBlk.fontData[i])) &&
-		    (sizeMatch (ctrlBlk, ctrlBlk.fontData[i])) &&
+		    (styleMatch(&ctrlBlk, ctrlBlk.fontData[i])) &&
+		    (sizeMatch (&ctrlBlk, ctrlBlk.fontData[i])) &&
 		    ((ctrlBlk.showPropFonts == PREF_PROP) || 
 		     (notPropFont(ctrlBlk.fontData[i]))))
 		{
@@ -541,8 +541,8 @@ static void	setupScrollLists(int dontChange, xfselControlBlkType ctrlBlk)
 		}
 
 		if ((dontChange != STYLE) &&
-		    (fontMatch(ctrlBlk, ctrlBlk.fontData[i])) &&
-		    (sizeMatch (ctrlBlk, ctrlBlk.fontData[i])) &&
+		    (fontMatch(&ctrlBlk, ctrlBlk.fontData[i])) &&
+		    (sizeMatch (&ctrlBlk, ctrlBlk.fontData[i])) &&
 		    ((ctrlBlk.showPropFonts == PREF_PROP) || 
 		     (notPropFont(ctrlBlk.fontData[i]))))
 		{
@@ -551,8 +551,8 @@ static void	setupScrollLists(int dontChange, xfselControlBlkType ctrlBlk)
 		}
 
 		if ((dontChange != SIZE) &&
-		    (fontMatch(ctrlBlk, ctrlBlk.fontData[i])) &&
-		    (styleMatch (ctrlBlk, ctrlBlk.fontData[i])) &&
+		    (fontMatch(&ctrlBlk, ctrlBlk.fontData[i])) &&
+		    (styleMatch (&ctrlBlk, ctrlBlk.fontData[i])) &&
 		    ((ctrlBlk.showPropFonts == PREF_PROP) || 
 		     (notPropFont(ctrlBlk.fontData[i]))))
 		{
@@ -563,7 +563,6 @@ static void	setupScrollLists(int dontChange, xfselControlBlkType ctrlBlk)
 	}	/* end - for (i = 0; i < ctrlBlk.numFonts; i++) */
 
 	/*	recreate all three scroll lists where necessary */
-
 	if (dontChange != FONT)
 	{
 		for (i = 0; i < itemCount1; i++)
@@ -649,16 +648,16 @@ static int	notPropFont(char *font)
 /*	returns TRUE if the style portion of the font matches the currently
 	selected style */
 
-static int	styleMatch(xfselControlBlkType ctrlBlk, char *font)
+static int	styleMatch(xfselControlBlkType *ctrlBlk, char *font)
 {
 	char	buff[TEMP_BUF_SIZE];
 
-	if (ctrlBlk.sel2 == NULL)
+	if (ctrlBlk->sel2 == NULL)
 		return(TRUE);
 
 	getStylePart(font, buff);
 
-	if (strcmp(buff, ctrlBlk.sel2) == 0)
+	if (strcmp(buff, ctrlBlk->sel2) == 0)
 		return(TRUE);
 	else
 		return(FALSE);
@@ -668,15 +667,15 @@ static int	styleMatch(xfselControlBlkType ctrlBlk, char *font)
 /*	returns TRUE if the size portion of the font matches the currently
 	selected size */
 
-static int	sizeMatch(xfselControlBlkType ctrlBlk, char *font)
+static int	sizeMatch(xfselControlBlkType *ctrlBlk, char *font)
 {
 	char	buff[TEMP_BUF_SIZE];
 
-	if (ctrlBlk.sel3 == NULL)
+	if (ctrlBlk->sel3 == NULL)
 		return(TRUE);
 
-	getSizePart(font, buff, ctrlBlk.showSizeInPixels);
-	if (strcmp(buff, ctrlBlk.sel3) == 0)
+	getSizePart(font, buff, ctrlBlk->showSizeInPixels);
+	if (strcmp(buff, ctrlBlk->sel3) == 0)
 		return(TRUE);
 	else
 		return(FALSE);
@@ -686,15 +685,15 @@ static int	sizeMatch(xfselControlBlkType ctrlBlk, char *font)
 /*	returns TRUE if the font portion of the font matches the currently
 	selected font */
 
-static int	fontMatch(xfselControlBlkType ctrlBlk, char *font)
+static int	fontMatch(xfselControlBlkType *ctrlBlk, char *font)
 {
 	char	buff[TEMP_BUF_SIZE];
 
-	if (ctrlBlk.sel1 == NULL)
+	if (ctrlBlk->sel1 == NULL)
 		return(TRUE);
 
 	getFontPart(font, buff);
-	if (strcmp(buff, ctrlBlk.sel1) == 0)
+	if (strcmp(buff, ctrlBlk->sel1) == 0)
 		return(TRUE);
 	else
 		return(FALSE);
@@ -875,9 +874,9 @@ static void	sizeToggleAction(Widget widget,
 		makeSelection = (ctrlBlk->sel3 != NULL);
 
 		for (i = 0; (makeSelection) && (i < ctrlBlk->numFonts); i++)
-			if ((fontMatch(*ctrlBlk, ctrlBlk->fontData[i])) &&
-			    (styleMatch(*ctrlBlk, ctrlBlk->fontData[i])) &&
-			    (sizeMatch(*ctrlBlk, ctrlBlk->fontData[i])))
+			if ((fontMatch(ctrlBlk, ctrlBlk->fontData[i])) &&
+			    (styleMatch(ctrlBlk, ctrlBlk->fontData[i])) &&
+			    (sizeMatch(ctrlBlk, ctrlBlk->fontData[i])))
 			{
 				getSizePart(ctrlBlk->fontData[i], newSize, 
 						    !ctrlBlk->showSizeInPixels);
@@ -1065,9 +1064,9 @@ static void	choiceMade(xfselControlBlkType *ctrlBlk)
 
 	for (i = 0; i < ctrlBlk->numFonts; i++)
 	{
-		if ((fontMatch(*ctrlBlk, ctrlBlk->fontData[i])) &&
-		    (styleMatch(*ctrlBlk, ctrlBlk->fontData[i])) &&
-		    (sizeMatch (*ctrlBlk, ctrlBlk->fontData[i])))
+		if ((fontMatch(ctrlBlk, ctrlBlk->fontData[i])) &&
+		    (styleMatch(ctrlBlk, ctrlBlk->fontData[i])) &&
+		    (sizeMatch (ctrlBlk, ctrlBlk->fontData[i])))
 		{
 			ctrlBlk->fontName = XtMalloc(
 					      strlen(ctrlBlk->fontData[i]) + 1);
@@ -1083,7 +1082,7 @@ static void	choiceMade(xfselControlBlkType *ctrlBlk)
 	}
  	else
 		DialogF (DF_ERR, ctrlBlk->form, 1, "Invalid Font Specification",
-								"Acknowledged");
+								     "Dismiss");
 }
 
 
@@ -1155,7 +1154,7 @@ static void	okAction(Widget widget, xfselControlBlkType *ctrlBlk,
 	if (i != 1)
 	{
 		DialogF (DF_ERR, ctrlBlk->okButton, 1, 
-				"Invalid Font Specification", "Acknowledged");
+				"Invalid Font Specification", "Dismiss");
 		XFreeFontNames(fontName);
 	}
 	else
