@@ -2,27 +2,28 @@
 *									       *
 * misc.c -- Miscelaneous Motif convenience functions			       *
 *									       *
-* Copyright (c) 1991 Universities Research Association, Inc.		       *
-* All rights reserved.							       *
+* Copyright (C) 1999 Mark Edel						       *
+*									       *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version.							               *
 * 									       *
-* This material resulted from work developed under a Government Contract and   *
-* is subject to the following license:  The Government retains a paid-up,      *
-* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
-* works, perform publicly and display publicly by or for the Government,       *
-* including the right to distribute to other Government contractors.  Neither  *
-* the United States nor the United States Department of Energy, nor any of     *
-* their employees, makes any warrenty, express or implied, or assumes any      *
-* legal liability or responsibility for the accuracy, completeness, or         *
-* usefulness of any information, apparatus, product, or process disclosed, or  *
-* represents that its use would not infringe privately owned rights.           *
-*                                        				       *
-* Fermilab Nirvana GUI Library						       *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* for more details.							       *
+* 									       *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA		                       *
+*									       *
+* Nirvana Text Editor	    						       *
 * July 28, 1992								       *
 *									       *
 * Written by Mark Edel							       *
 *									       *
 *******************************************************************************/
-static char SCCSID[] = "@(#)misc.c	1.39	2/1/96";
 #include <stdlib.h>
 #include <stdarg.h>
 #include <math.h>
@@ -35,6 +36,7 @@ static char SCCSID[] = "@(#)misc.c	1.39	2/1/96";
 #endif /*VMS*/
 #include <X11/Intrinsic.h>
 #include <X11/Xatom.h>
+#include <X11/keysym.h>
 #include <Xm/Xm.h>
 #include <Xm/Label.h>
 #include <Xm/LabelG.h>
@@ -53,6 +55,20 @@ static char SCCSID[] = "@(#)misc.c	1.39	2/1/96";
    POSIX compliance is turned on */
 extern double strtod();
 
+/* structure for passing history-recall data to callbacks */
+typedef struct {
+    char ***list;
+    int *nItems;
+    int index;
+} histInfo;
+
+/* Maximum size of a history-recall list.  Typically never invoked, since
+   user must first make this many entries in the text field, limited for
+   safety, to the maximum reasonable number of times user can hit up-arrow
+   before carpal tunnel syndrome sets in */
+#define HISTORY_LIST_TRIM_TO 3 /* 1000 */
+#define HISTORY_LIST_MAX 6 /* 2000 */
+
 /* flags to enable/disable delete key remapping and pointer centered dialogs */
 static int RemapDeleteEnabled = True;
 static int PointerCenteredDialogsEnabled = False;
@@ -82,6 +98,9 @@ static void removeWhiteSpace(char *string);
 static void warnHandlerCB(String message);
 static void passwdCB(Widget w, char * passTxt, XmTextVerifyCallbackStruct
 	*txtVerStr);
+static void histDestroyCB(Widget w, XtPointer clientData, XtPointer callData);
+static void histArrowKeyEH(Widget w, XtPointer callData, XEvent *event,
+	Boolean *continueDispatch);
 
 /*
 ** Set up closeCB to be called when the user selects close from the
@@ -152,6 +171,54 @@ void SetDeleteRemap(int state)
     RemapDeleteEnabled = state;
 }
 
+/*
+** This routine resolves a window manager protocol incompatibility between
+** the X toolkit and several popular window managers.  Using this in place
+** of XtRealizeWidget will realize the window in a way which allows the
+** affected window managers to apply their own placement strategy to the
+** window, as opposed to forcing the window to a specific location.
+**
+** One of the hints in the WM_NORMAL_HINTS protocol, PPlacement, gets set by
+** the X toolkit (probably part of the Core or Shell widget) when a shell
+** widget is realized to the value stored in the XmNx and XmNy resources of the
+** Core widget.  While callers can set these values, there is no "unset" value
+** for these resources.  On systems which are more Motif aware, a PPosition
+** hint of 0,0, which is the default for XmNx and XmNy, is interpreted as,
+** "place this as if no hints were specified".  Unfortunately the fvwm family
+** of window managers, which are now some of the most popular, interpret this
+** as "place this window at (0,0)".  This routine intervenes between the
+** realizing and the mapping of the window to remove the inappropriate
+** PPlacement hint.
+*/ 
+void RealizeWithoutForcingPosition(Widget shell)
+{
+    XSizeHints *hints = XAllocSizeHints();
+    long suppliedHints;
+    Boolean mappedWhenManaged;
+    
+    /* Temporarily set value of XmNmappedWhenManaged
+       to stop the window from popping up right away */
+    XtVaGetValues(shell, XmNmappedWhenManaged, &mappedWhenManaged, 0);
+    XtVaSetValues(shell, XmNmappedWhenManaged, False, 0);
+    
+    /* Realize the widget in unmapped state */
+    XtRealizeWidget(shell);
+    
+    /* Get rid of the incorrect WMNormal hint */
+    if (XGetWMNormalHints(XtDisplay(shell), XtWindow(shell), hints,
+    	    &suppliedHints)) {
+	hints->flags &= ~PPosition;
+	XSetWMNormalHints(XtDisplay(shell), XtWindow(shell), hints);
+    }
+    XFree(hints);
+    
+    /* Map the widget */
+    XtMapWidget(shell);
+    
+    /* Restore the value of XmNmappedWhenManaged */
+    XtVaSetValues(shell, XmNmappedWhenManaged, mappedWhenManaged, 0);
+}
+
 /*
 ** ManageDialogCenteredOnPointer is used in place of XtManageChild for
 ** popping up a dialog to enable the dialog to be centered under the
@@ -224,6 +291,37 @@ void SetPointerCenteredDialogs(int state)
     PointerCenteredDialogsEnabled = state;
 }
 
+
+/*
+** Raise a window to the top and give it the input focus.  Setting input focus
+** is important on systems which use explict (rather than pointer) focus.
+**
+** The X alternatives XMapRaised, and XSetInputFocus both have problems.
+** XMapRaised only gives the window the focus if it was initially not visible,
+** and XSetInputFocus sets the input focus, but crashes if the window is not
+** visible.
+**
+** This routine should also be used in the case where a dialog is popped up and
+** subsequent calls to the dialog function use a flag, or the XtIsManaged, to
+** decide whether to create a new instance of the dialog, because on slower
+** systems, events can intervene while a dialog is still on its way up and its
+** window is still invisible, causing a subtle crash potential if
+** XSetInputFocus is used.
+*/
+void RaiseShellWindow(Widget shell)
+{
+    RaiseWindow(XtDisplay(shell), XtWindow(shell));
+}
+void RaiseWindow(Display *display, Window w)
+{
+    XWindowAttributes winAttr;
+
+    XGetWindowAttributes(display, w, &winAttr);
+    if (winAttr.map_state == IsViewable)
+    	XSetInputFocus(display, w, RevertToParent, CurrentTime);	
+    XMapRaised(display, w);
+}
+
 /*
 ** Add a handler for mnemonics in a dialog (Motif currently only handles
 ** mnemonics in menus) following the example of M.S. Windows.  To add
@@ -555,11 +653,11 @@ int GetFloatTextWarn(Widget text, double *value, char *fieldName, int warnBlank)
     valueStr = XmTextGetString(text);
     if (result == TEXT_IS_BLANK)
     	DialogF (DF_ERR, text, 1, "Please supply %s value",
-    		 "Acknowledged", fieldName);
+    		 "Dismiss", fieldName);
     else /* TEXT_NOT_NUMBER */
     	DialogF (DF_ERR, text, 1,
     		 "Can't read %s value: \"%s\"",
-    		 "Acknowledged", fieldName, valueStr);
+    		 "Dismiss", fieldName, valueStr);
     XtFree(valueStr);
     return result;
 }
@@ -574,15 +672,136 @@ int GetIntTextWarn(Widget text, int *value, char *fieldName, int warnBlank)
     valueStr = XmTextGetString(text);
     if (result == TEXT_IS_BLANK)
     	DialogF (DF_ERR, text, 1, "Please supply a value for %s",
-    		 "Acknowledged", fieldName);
+    		 "Dismiss", fieldName);
     else /* TEXT_NOT_NUMBER */
     	DialogF (DF_ERR, text, 1,
     		 "Can't read integer value \"%s\" in %s",
-    		 "Acknowledged", valueStr, fieldName);
+    		 "Dismiss", valueStr, fieldName);
     XtFree(valueStr);
     return result;
 }
 
+int TextWidgetIsBlank(Widget textW)
+{
+    char *str;
+    int retVal;
+    
+    str = XmTextGetString(textW);
+    removeWhiteSpace(str);
+    retVal = *str == '\0';
+    XtFree(str);
+    return retVal;
+}
+
+/*
+** Turn a multi-line editing text widget into a fake single line text area
+** by disabling the translation for Return.  This is a way to give users
+** extra space, by allowing wrapping, but still prohibiting newlines.
+** (SINGLE_LINE_EDIT mode can't be used, in this case, because it forces
+** the widget to be one line high).
+*/
+void MakeSingleLineTextW(Widget textW)
+{
+    static XtTranslations noReturnTable = NULL;
+    static char *noReturnTranslations = "<Key>Return: activate()\n";
+    
+    if (noReturnTable == NULL)
+    	noReturnTable = XtParseTranslationTable(noReturnTranslations);
+    XtOverrideTranslations(textW, noReturnTable);
+}
+
+/*
+** Add up-arrow/down-arrow recall to a single line text field.  When user
+** presses up-arrow, string is cleared and recent entries are presented,
+** moving to older ones as each successive up-arrow is pressed.  Down-arrow
+** moves to more recent ones, final down-arrow clears the field.  Associated
+** routine, AddToHistoryList, makes maintaining a history list easier.
+**
+** Arguments are the widget, and pointers to the history list and number of
+** items, which are expected to change periodically.
+*/
+void AddHistoryToTextWidget(Widget textW, char ***historyList, int *nItems)
+{
+    histInfo *histData;
+    
+    /* create a data structure for passing history info to the callbacks */
+    histData = (histInfo *)XtMalloc(sizeof(histInfo));
+    histData->list = historyList;
+    histData->nItems = nItems;
+    histData->index = -1;
+    
+    /* Add an event handler for handling up/down arrow events */
+    XtAddEventHandler(textW, KeyPressMask, False,
+    	    (XtEventHandler)histArrowKeyEH, histData);
+    
+    /* Add a destroy callback for freeing history data structure */
+    XtAddCallback(textW, XmNdestroyCallback, histDestroyCB, histData);
+}
+
+static void histDestroyCB(Widget w, XtPointer clientData, XtPointer callData)
+{
+    XtFree((char *)clientData);
+}
+
+static void histArrowKeyEH(Widget w, XtPointer callData, XEvent *event,
+	Boolean *continueDispatch)
+{
+    histInfo *histData = (histInfo *)callData;
+    KeySym keysym = XLookupKeysym((XKeyEvent *)event, 0);
+    
+    /* only process up and down arrow keys */
+    if (keysym != XK_Up && keysym != XK_Down)
+    	return;
+    
+    /* increment or decrement the index depending on which arrow was pressed */
+    histData->index += (keysym == XK_Up) ? 1 : -1;
+
+    /* if the index is out of range, beep, fix it up, and return */
+    if (histData->index < -1) {
+    	histData->index = -1;
+	XBell(XtDisplay(w), 0);
+    	return;
+    }
+    if (histData->index >= *histData->nItems) {
+    	histData->index = *histData->nItems - 1;
+	XBell(XtDisplay(w), 0);
+    	return;
+    }
+    
+    /* Change the text field contents */
+    XmTextSetString(w, histData->index == -1 ? "" :
+	    (*histData->list)[histData->index]);
+}
+
+/*
+** Copies a string on to the end of history list, which may be reallocated
+** to make room.  If historyList grows beyond its internally set boundary
+** for size (HISTORY_LIST_MAX), it is trimmed back to a smaller size
+** (HISTORY_LIST_TRIM_TO).  Before adding to the list, checks if the item
+** is a duplicate of the last item.  If so, it is not added.
+*/	
+void AddToHistoryList(char *newItem, char ***historyList, int *nItems)
+{
+    char **newList;
+    int i;
+    
+    if (*nItems != 0 && !strcmp(newItem, **historyList))
+	return;
+    if (*nItems == HISTORY_LIST_MAX) {
+	for (i=HISTORY_LIST_TRIM_TO; i<HISTORY_LIST_MAX; i++)
+	    XtFree((*historyList)[i]);
+	*nItems = HISTORY_LIST_TRIM_TO;
+    }
+    newList = (char **)XtMalloc(sizeof(char *) * (*nItems + 1));
+    for (i=0; i < *nItems; i++)
+	newList[i+1] = (*historyList)[i];
+    if (*nItems != 0 && *historyList != NULL)
+	XtFree((char *)*historyList);
+    (*nItems)++;
+    newList[0] = XtNewString(newItem);
+    *historyList = newList;
+}
+
 /*
  * PasswordText - routine to add a callback to any text widget so that all
  * 		  text typed by the user is echoed with asterisks, allowing
@@ -643,12 +862,52 @@ void EndWait(Widget topCursorWidget)
     XUndefineCursor(XtDisplay(topCursorWidget), XtWindow(topCursorWidget));
 }
 
+/*
+** Create an X window geometry string from width, height, x, and y values.
+** This is a complement to the X routine XParseGeometry, and uses the same
+** bitmask values (XValue, YValue, WidthValue, HeightValue, XNegative, and
+** YNegative) as defined in <X11/Xutil.h> and documented under XParseGeometry.
+** It expects a string of at least MAX_GEOMETRY_STRING_LEN in which to write
+** result.  Note that in a geometry string, it is not possible to supply a y
+** position without an x position.  Also note that the X/YNegative flags
+** mean "add a '-' and negate the value" which is kind of odd.
+*/
+void CreateGeometryString(char *string, short x, short y,
+	short width, short height, int bitmask)
+{
+    char *ptr = string;
+    int nChars;
+    
+    if (bitmask & WidthValue) {
+    	sprintf(ptr, "%d%n", width, &nChars);
+	ptr += nChars;
+    }
+    if (bitmask & HeightValue) {
+	sprintf(ptr, "x%d%n", height, &nChars);
+	ptr += nChars;
+    }
+    if (bitmask & XValue) {
+	if (bitmask & XNegative)
+    	    sprintf(ptr, "-%d%n", -x, &nChars);
+	else
+    	    sprintf(ptr, "+%d%n", x, &nChars);
+	ptr += nChars;
+    }
+    if (bitmask & YValue) {
+	if (bitmask & YNegative)
+    	    sprintf(ptr, "-%d%n", -y, &nChars);
+	else
+    	    sprintf(ptr, "+%d%n", y, &nChars);
+	ptr += nChars;
+    }
+    *ptr = '\0';
+}
+
 /*									     */
 /* passwdCB: callback routine added by PasswordText routine.  This routine   */
 /* 	     echoes each character typed as an asterisk (*) and a few other  */
 /* 	     necessary things so that the password typed in is not visible   */
 /*									     */
-
 static void passwdCB(Widget w, char * passTxt, XmTextVerifyCallbackStruct
 	*txtVerStr)
 
@@ -721,14 +980,13 @@ static void removeWhiteSpace(char *string)
     char *outPtr = string;
     
     while (TRUE) {
-    	if (*string != ' ' && *string != '\t')
-	    *(outPtr++) = *(string++);
-	else
-	    string++;
     	if (*string == 0) {
 	    *outPtr = 0;
 	    return;
-	}
+    	} else if (*string != ' ' && *string != '\t')
+	    *(outPtr++) = *(string++);
+	else
+	    string++;
     }
 }
 
